@@ -7,6 +7,7 @@
  * this program the details of the active license.
  */
 use crate::error::{Result, ResultWithContext, VegaFusionError};
+use crate::spec::axis::AxisSpec;
 use crate::spec::chart::{ChartVisitor, MutChartVisitor};
 use crate::spec::data::DataSpec;
 use crate::spec::scale::ScaleSpec;
@@ -46,6 +47,9 @@ pub struct MarkSpec {
     #[serde(default, skip_serializing_if = "Vec::is_empty")]
     pub scales: Vec<ScaleSpec>,
 
+    #[serde(default, skip_serializing_if = "Vec::is_empty")]
+    pub axes: Vec<AxisSpec>,
+
     #[serde(default, skip_serializing_if = "Vec::is_empty")]
     pub transform: Vec<Value>,
 
@@ -66,6 +70,9 @@ impl MarkSpec {
         for scale in &self.scales {
             visitor.visit_scale(scale, &scope)?;
         }
+        for axis in &self.axes {
+            visitor.visit_axis(axis, &scope)?;
+        }
         for signal in &self.signals {
             visitor.visit_signal(signal, &scope)?;
         }
@@ -97,6 +104,9 @@ impl MarkSpec {
         for scale in &mut self.scales {
             visitor.visit_scale(scale, &scope)?;
         }
+        for axis in &mut self.axes {
+            visitor.visit_axis(axis, &scope)?;
+        }
         for signal in &mut self.signals {
             visitor.visit_signal(signal, &scope)?;
         }
