@@ -14,6 +14,7 @@ pub mod filter;
 pub mod formula;
 pub mod joinaggregate;
 pub mod lookup;
+pub mod project;
 pub mod sequence;
 pub mod timeunit;
 pub mod unsupported;
@@ -22,16 +23,20 @@ pub mod window;
 use crate::spec::transform::{extent::ExtentTransformSpec, filter::FilterTransformSpec};
 
 use crate::error::Result;
+use crate::expression::column_usage::{ColumnUsage, DatasetsColumnUsage, VlSelectionFields};
 use crate::spec::transform::aggregate::AggregateTransformSpec;
 use crate::spec::transform::bin::BinTransformSpec;
 use crate::spec::transform::collect::CollectTransformSpec;
 use crate::spec::transform::formula::FormulaTransformSpec;
 use crate::spec::transform::joinaggregate::JoinAggregateTransformSpec;
 use crate::spec::transform::lookup::LookupTransformSpec;
+use crate::spec::transform::project::ProjectTransformSpec;
 use crate::spec::transform::sequence::SequenceTransformSpec;
 use crate::spec::transform::timeunit::TimeUnitTransformSpec;
 use crate::spec::transform::unsupported::*;
 use crate::spec::transform::window::WindowTransformSpec;
+use crate::task_graph::graph::ScopedVariable;
+use crate::task_graph::scope::TaskScope;
 use crate::task_graph::task::InputVariable;
 use serde::{Deserialize, Serialize};
 use std::ops::Deref;
@@ -48,6 +53,7 @@ pub enum TransformSpec {
     Timeunit(TimeUnitTransformSpec),
     JoinAggregate(JoinAggregateTransformSpec),
     Window(WindowTransformSpec),
+    Project(ProjectTransformSpec),
 
     // Unsupported
     CountPattern(CountpatternTransformSpec),
@@ -79,7 +85,6 @@ pub enum TransformSpec {
     Partition(PartitionTransformSpec),
     Pie(PieTransformSpec),
     Pivot(PivotTransformSpec),
-    Project(ProjectTransformSpec),
     Quantile(QuantileTransformSpec),
     Regression(RegressionTransformSpec),
     ResolveFilter(ResolvefilterTransformSpec),
@@ -106,6 +111,7 @@ impl Deref for TransformSpec {
             TransformSpec::Aggregate(t) => t,
             TransformSpec::Collect(t) => t,
             TransformSpec::Timeunit(t) => t,
+            TransformSpec::Project(t) => t,
 
             // Supported for dependency determination, not implementation
             TransformSpec::Lookup(t) => t,
@@ -141,7 +147,6 @@ impl Deref for TransformSpec {
             TransformSpec::Partition(t) => t,
             TransformSpec::Pie(t) => t,
             TransformSpec::Pivot(t) => t,
-            TransformSpec::Project(t) => t,
             TransformSpec::Quantile(t) => t,
             TransformSpec::Regression(t) => t,
             TransformSpec::ResolveFilter(t) => t,
@@ -170,4 +175,31 @@ pub trait TransformSpecTrait {
     fn input_vars(&self) -> Result<Vec<InputVariable>> {
         Ok(Default::default())
     }
+
+    fn transform_columns(
+        &self,
+        _datum_var: &Option<ScopedVariable>,
+        _usage_scope: &[u32],
+        _task_scope: &TaskScope,
+        _vl_selection_fields: &VlSelectionFields,
+    ) -> TransformColumns {
+        TransformColumns::Unknown
+    }
+}
+
+pub enum TransformColumns {
+    /// Transforms that pass through existing columns and produce zero or more new columns
+    PassThrough {
+        usage: DatasetsColumnUsage,
+        produced: ColumnUsage,
+    },
+
+    /// Transforms that overwrite all input columns, leaving only those produced by the transform
+    Overwrite {
+        usage: DatasetsColumnUsage,
+        produced: ColumnUsage,
+    },
+
+    /// Transforms with unknown usage and/or production of columns
+    Unknown,
 }
